Package: EnrichIntersect
Type: Package
Title: Enrichment Analysis and Intersecting Sankey Diagram
Version: 1.0
URL: https://github.com/ocbe-uio/EnrichIntersect
BugReports: https://github.com/ocbe-uio/EnrichIntersect/issues
Authors@R: c(person("Zhi", "Zhao", email = "zhi.zhao@medisin.uio.no", role = c("aut","cre")), person("Manuela", "Zucknick", email = "manuela.zucknick@medisin.uio.no", role = c("aut")), person("Tero", "Aittokallio", email = "t.a.aittokallio@medisin.uio.no", role = c("ctb")))
Description: A flexible tool for enrichment analysis based on user-defined sets. It allows users to perform over-representation analysis of the custom sets among any specified ranked feature list, hence making enrichment analysis applicable to various types of data from different scientific fields. 'EnrichIntersect' also enables an interactive means to visualize identified associations based on, for example, the mix-lasso model (Zhao et al., 2022 <doi:10.1016/j.isci.2022.104767>) or similar methods.
License: MIT + file LICENSE
Depends: R (>= 4.0)
Encoding: UTF-8
LinkingTo: Rcpp, RcppArmadillo
Imports: Rcpp, ggplot2, dplyr, networkD3, jsonlite, htmlwidgets,
        webshot2, stats, methods, grDevices
Suggests: knitr
LazyData: true
VignetteBuilder: knitr
NeedsCompilation: yes
Config/roxygen2/version: 8.0.0
Packaged: 2026-05-30 00:19:06 UTC; zhiz
Author: Zhi Zhao [aut, cre],
  Manuela Zucknick [aut],
  Tero Aittokallio [ctb]
Maintainer: Zhi Zhao <zhi.zhao@medisin.uio.no>
Repository: CRAN
Date/Publication: 2026-05-30 02:10:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-30 07:03:05 UTC; unix
Archs: EnrichIntersect.so.dSYM

Package: EnrichIntersect
Type: Package
Title: Enrichment Analysis and Intersecting Sankey Diagram
Version: 1.0
URL: https://github.com/ocbe-uio/EnrichIntersect
BugReports: https://github.com/ocbe-uio/EnrichIntersect/issues
Authors@R: c(person("Zhi", "Zhao", email = "zhi.zhao@medisin.uio.no", role = c("aut","cre")), person("Manuela", "Zucknick", email = "manuela.zucknick@medisin.uio.no", role = c("aut")), person("Tero", "Aittokallio", email = "t.a.aittokallio@medisin.uio.no", role = c("ctb")))
Description: A flexible tool for enrichment analysis based on user-defined sets. It allows users to perform over-representation analysis of the custom sets among any specified ranked feature list, hence making enrichment analysis applicable to various types of data from different scientific fields. 'EnrichIntersect' also enables an interactive means to visualize identified associations based on, for example, the mix-lasso model (Zhao et al., 2022 <doi:10.1016/j.isci.2022.104767>) or similar methods.
License: MIT + file LICENSE
Depends: R (>= 4.0)
Encoding: UTF-8
LinkingTo: Rcpp, RcppArmadillo
Imports: Rcpp, ggplot2, dplyr, networkD3, jsonlite, htmlwidgets,
        webshot2, stats, methods, grDevices
Suggests: knitr
LazyData: true
VignetteBuilder: knitr
NeedsCompilation: yes
Config/roxygen2/version: 8.0.0
Packaged: 2026-05-30 00:19:06 UTC; zhiz
Author: Zhi Zhao [aut, cre],
  Manuela Zucknick [aut],
  Tero Aittokallio [ctb]
Maintainer: Zhi Zhao <zhi.zhao@medisin.uio.no>
Repository: CRAN
Date/Publication: 2026-05-30 02:10:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-30 02:42:21 UTC; unix
Archs: EnrichIntersect.so.dSYM

# Generated by roxygen2: do not edit by hand

export(enrichment)
export(intersectSankey)
import(ggplot2)
importFrom(Rcpp,sourceCpp)
importFrom(dplyr,"%>%")
importFrom(grDevices,hcl.colors)
importFrom(htmlwidgets,onRender)
importFrom(htmlwidgets,saveWidget)
importFrom(jsonlite,toJSON)
importFrom(methods,hasArg)
importFrom(networkD3,JS)
importFrom(networkD3,sankeyNetwork)
importFrom(stats,p.adjust)
importFrom(webshot2,webshot)
useDynLib(EnrichIntersect, .registration = TRUE)

# EnrichIntersect

[![CRAN_Status_Badge](http://www.r-pkg.org/badges/version/EnrichIntersect)](https://cran.r-project.org/package=EnrichIntersect)

This is a flexible R package for enrichment analysis based on user-defined sets. It allows users to perform over-representation analysis of the custom sets among any specified ranked feature list, hence making enrichment analysis applicable to various types of data from different scientific fields. EnrichIntersect also enables an interactive means to visualize identified associations based on, for example, the mix-lasso model ([Zhao et al., 2022](https://doi.org/10.1016/j.isci.2022.104767)) or similar methods. 

## Installation

Install the latest released version from [CRAN](https://CRAN.R-project.org/package=EnrichIntersect)

```r
install.packages("EnrichIntersect")
library("EnrichIntersect")
```

Install the latest development version from GitHub

```r
#install.packages("remotes")
remotes::install_github("ocbe-uio/EnrichIntersect")
library("EnrichIntersect")
```

## Examples

### Plot enrichment map

Data set `cancers_drug_groups` is a list including a score dataframe with 147 drugs as rows and 19 cancer types as columns, and a dataframe with nice pre-defined drug groups (1st column) of the 147 drugs (2nd column).

```r
data(cancers_drug_groups, package = "EnrichIntersect")
x <- cancers_drug_groups$score
custom.set <- cancers_drug_groups$custom.set
set.seed(123)
enrich <- enrichment(x, custom.set, permute.n = 5)
```

<img src="man/figures/plot_enrich.png" width="90%" />

### Plot Sankey diagram for intersecting set through an array

Data set `cancers_genes_drugs` is an array with association scores between 56 genes (1st dimension), three cancer types (2nd dimension) and two drugs (3rd dimension).

```r
data(cancers_genes_drugs, package = "EnrichIntersect")
# better to use argument `out.fig = "pdf"` for printing a pdf or html figure
intersectSankey(cancers_genes_drugs, step.names = c("Cancers","Genes","Drugs"))
```

<img src="man/figures/plot_sankey.png" width="90%" />



## Citation

> Zhi Zhao, Manuela Zucknick, Tero Aittokallio (2022).
> EnrichIntersect: an R package for custom set enrichment analysis and
> interactive visualization of intersecting sets. _Bioinformatics Advances_,
> vbac073. DOI: [10.1093/bioadv/vbac073](https://doi.org/10.1093/bioadv/vbac073).

> Zhi Zhao, Shixiong Wang, Manuela Zucknick, Tero Aittokallio (2022).
> Tissue-specific identification of multi-omics features for pan-cancer
> drug response prediction. _iScience_, 25(8): 104767. DOI:
> [10.1016/j.isci.2022.104767](https://doi.org/10.1016/j.isci.2022.104767).

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

## ----eval=TRUE----------------------------------------------------------------
library("EnrichIntersect")

## ----enrichment_map, results="hide", fig.width=7.2, fig.height=5--------------
data(cancers_drug_groups, package = "EnrichIntersect")
x <- cancers_drug_groups$score
custom.set <- cancers_drug_groups$custom.set
set.seed(123)
enrich <- enrichment(x, custom.set, permute.n = 1000)

## ----sankey_diagram, fig.width=9, fig.height=7--------------------------------
data(cancers_genes_drugs, package = "EnrichIntersect")
# better to use argument `out.fig = "pdf"` for printing a pdf or html figure
intersectSankey(cancers_genes_drugs, step.names = c("Cancers", "Genes", "Drugs"))

#' @title Plot enrichment map
#' @description
#' Plot enrichment map through a vector or matrix of scores and a self-defined
#' set that summarizes a few groups of the names of the vector or matrix.
#'
#' @name enrichment
#'
#' @import ggplot2
#' @importFrom stats p.adjust
#'
#' @param x a vector or matrix of scores to be enriched
#' @param custom.set a self-defined set. The first column contains feature names,
#'   and the second column, preferably named "group", contains group names.
#' @param alpha exponent weight of the score of ordered features.
#' @param normalize logical value to determine if normalizing enrichment scores.
#' @param permute.n number of custom-set permutations for significance testing.
#' @param padj.method correction method passed to stats::p.adjust.
#' @param pvalue.cutoff cutoff for both unadjusted and adjusted p-value.
#' @param angle angle of rotating x-axis labels.
#' @param match.feature one of "rownames" or "colnames". Default is "rownames",
#'   which keeps the original EnrichIntersect behavior.
#' @param ... other arguments
#'
#' @return A list including S, pvalue, and g.
#'
#' @export
enrichment <- function(
    x,
    custom.set,
    alpha = 0,
    normalize = TRUE,
    permute.n = 100,
    padj.method = "none",
    pvalue.cutoff = 0.05,
    angle = 45,
    match.feature = c("rownames", "colnames"),
    ...
) {
  match.feature <- match.arg(match.feature)
  
  ## --------------------------------------------------------------------------
  ## Original-style input handling
  ## --------------------------------------------------------------------------
  
  if (is.matrix(x) || is.data.frame(x)) {
    if (any(colSums(is.na(x)) == nrow(x)) && ncol(x) > 1) {
      stop("The argument 'x' matrix has some columns with all missing values!")
    }
  } else {
    x <- as.matrix(x)
  }
  
  x <- as.matrix(x)
  storage.mode(x) <- "double"
  
  if (is.null(colnames(x))) {
    colnames(x) <- "X"
  }
  
  if (is.null(rownames(x))) {
    stop("The argument 'x' must have rownames matching the feature names in 'custom.set'!")
  }
  
  if (is.matrix(custom.set) || is.data.frame(custom.set)) {
    if (dim(custom.set)[2] != 2) {
      stop("The argument 'custom.set' has to have two columns!")
    }
  } else {
    stop("The argument 'custom.set' has to be a matrix or dataframe!")
  }
  
  custom.set <- as.data.frame(custom.set, stringsAsFactors = FALSE)
  
  ## Original code uses custom.set$group.
  ## If the second column is not named "group", name it "group" for compatibility.
  if (!"group" %in% colnames(custom.set)) {
    colnames(custom.set)[2] <- "group"
  }
  
  custom.set[[1]] <- trimws(as.character(custom.set[[1]]))
  custom.set$group <- trimws(as.character(custom.set$group))
  
  ## --------------------------------------------------------------------------
  ## Support original orientation and optional transposed orientation
  ##
  ## Original behavior:
  ##   features are rownames(x)
  ##
  ## Optional:
  ##   features are colnames(x), so use t(x) internally
  ## --------------------------------------------------------------------------
  
  if (match.feature == "colnames") {
    if (is.null(colnames(x))) {
      stop("When match.feature = 'colnames', x must have colnames.")
    }
    
    x_work <- t(x)
  } else {
    x_work <- x
  }
  
  if (anyDuplicated(rownames(x_work))) {
    stop("Feature names must be unique.")
  }
  
  features <- intersect(rownames(x_work), custom.set[[1]])
  
  if (length(features) == 0L) {
    stop("None of the features in 'custom.set' matched the feature names of 'x'.")
  }
  
  groups <- unique(custom.set$group[custom.set[[1]] %in% features])
  n_groups <- length(groups)
  
  if (n_groups == 0L) {
    stop("No valid groups were found in 'custom.set'.")
  }
  
  ## Restrict x to matched features, preserving intersect() order.
  x_work <- x_work[features, , drop = FALSE]
  
  ## Build group-wise feature index list for the C++ core.
  set_indices <- lapply(groups, function(g) {
    idx <- match(
      custom.set[[1]][custom.set$group == g],
      features,
      nomatch = 0L
    )
    
    as.integer(unique(idx[idx > 0L]))
  })
  
  names(set_indices) <- groups
  
  ## --------------------------------------------------------------------------
  ## RcppArmadillo replacement for original heavy for-loop
  ##
  ## C++ returns matrices with:
  ##   rows    = columns/profiles of x_work
  ##   columns = custom-set groups
  ##
  ## This matches the original S and pvalue orientation before t().
  ## --------------------------------------------------------------------------
  
  core <- enrichment_core_original(
    x = x_work,
    set_indices = set_indices,
    alpha = alpha,
    normalize = normalize,
    permute_n = as.integer(permute.n)
  )
  
  S <- core$S
  pvalue <- core$pvalue
  
  rownames(S) <- colnames(x_work)
  colnames(S) <- groups
  
  rownames(pvalue) <- colnames(x_work)
  colnames(pvalue) <- groups
  
  ## --------------------------------------------------------------------------
  ## Original post-processing
  ## --------------------------------------------------------------------------
  
  pvalue[is.na(S)] <- NA
  pvalue <- t(pvalue)
  
  if (!padj.method == "none") {
    pvalue <- p.adjust(pvalue, method = padj.method)
  }
  
  S <- t(S)
  
  ## Define plotting dataframe
  dat <- data.frame(
    x = factor(rep(colnames(S), each = nrow(S))),
    y = rep(rownames(S), ncol(S)),
    ks = as.vector(S),
    pvalue = as.vector(pvalue)
  )
  
  dat$y <- factor(
    dat$y,
    levels = levels(factor(dat$y))[c(length(unique(dat$y)):1)]
  )
  
  ks.min <- min(dat$ks, na.rm = TRUE)
  
  dat$ks[dat$ks < 0 & !is.na(dat$ks)] <- ks.min - 0.001
  dat$ks <- dat$ks - (ks.min - 0.001)
  
  if (sum(!is.na(dat$ks)) >= 2L) {
    dat$ks[which.max(dat$ks)] <- round(sort(dat$ks, decreasing = TRUE)[2] + 0.5)
  }
  
  dat$border <- rep("red", nrow(dat))
  dat$border[dat$pvalue >= pvalue.cutoff] <- "gray"
  
  y <- border <- ks <- NULL
  
  if (normalize) {
    ES.name <- "Normalized\nEnrichment\nScore"
  } else {
    ES.name <- "Enrichment\nScore"
  }
  
  ## --------------------------------------------------------------------------
  ## Original plot format, with legend order:
  ##
  ##   1. significance legend, e.g. p<0.05
  ##   2. p-value colorbar
  ##   3. enrichment score size legend
  ## --------------------------------------------------------------------------
  
  if (any(dat$pvalue < pvalue.cutoff, na.rm = TRUE)) {
    g <- ggplot(data = dat) +
      geom_point(
        aes(x = x, y = y, color = border, fill = pvalue, size = ks),
        shape = 21
      ) +
      scale_fill_gradientn(
        name = "p-value",
        na.value = "black",
        colours = c("blue", "white"),
        limits = c(0, 1),
        guide = guide_colorbar(
          barheight = 3,
          barwidth = 1,
          order = 2
        )
      ) +
      geom_point(
        aes(x = x, y = y, size = ks),
        color = dat$border,
        shape = 21
      ) +
      guides(
        colour = guide_legend(
          override.aes = list(size = 5),
          order = 1
        )
      ) +
      scale_size(
        name = ES.name,
        range = c(1, 5),
        breaks = c(0, 1, 2, 3),
        guide = guide_legend(
          keyheight = .8,
          order = 3
        )
      ) +
      theme(
        axis.text.x = element_text(
          size = 8,
          angle = angle,
          vjust = 1,
          hjust = 1
        ),
        legend.margin = margin(-0.1, 0, 0, 0, unit = "cm"),
        legend.box = "vertical"
      ) +
      xlab("") +
      ylab("")
    
    if (pvalue.cutoff == 0.05) {
      g <- g +
        scale_color_manual(
          name = NULL,
          values = c(`red` = "red"),
          labels = "p<0.05",
          guide = guide_legend(order = 1)
        )
    } else {
      if (pvalue.cutoff == 0.1) {
        g <- g +
          scale_color_manual(
            name = NULL,
            values = c(`red` = "red"),
            labels = "p<0.1",
            guide = guide_legend(order = 1)
          )
      } else {
        g <- g +
          scale_color_manual(
            name = NULL,
            values = c("gray", "red"),
            labels = paste0("p", c(">=", "<"), pvalue.cutoff),
            guide = guide_legend(order = 1)
          )
      }
    }
  } else {
    g <- ggplot(data = dat) +
      geom_point(
        aes(x = x, y = y, fill = pvalue, size = ks),
        shape = 21
      ) +
      scale_fill_gradientn(
        name = "p-value",
        na.value = "black",
        colours = c("blue", "white"),
        limits = c(round(min(dat$pvalue, na.rm = TRUE), 2), 1),
        guide = guide_colorbar(
          barheight = 3,
          barwidth = 1,
          order = 1
        )
      ) +
      scale_size(
        name = ES.name,
        range = c(1, 5),
        breaks = c(0, 1, 2, 3),
        guide = guide_legend(
          keyheight = .8,
          order = 2
        )
      ) +
      theme(
        axis.text.x = element_text(
          size = 8,
          angle = 45,
          vjust = 1,
          hjust = 1
        ),
        legend.box = "vertical"
      ) +
      xlab("") +
      ylab("")
  }
  
  print(g)
  
  return(
    list(
      S = S,
      pvalue = pvalue,
      g = g
    )
  )
}

#' @title Data set 'cancers_genes_drugs'
#'
#' @description
#' Data set 'cancers_genes_drugs' is an array with association scores between 
#' 56 genes (1st dimension), three cancer types (2nd dimension) and two drugs 
#' (3rd dimension). More details of the data can be found in the results of 
#' paper 'Tissue-specific identification of multi-omics features for pan-cancer 
#' drug response prediction' by Zhao Z., Wang S. Zucknick M. and Aittokallio 
#' T. (2022).
#'
#' @return No return value.
#'
#' @references Zhao Z, Wang S, Zucknick M and Aittokallio T (2022). \emph{
#' Tissue-specific identification of multi-omics features for pan-cancer drug 
#' response prediction}. iScience, 25(8): 104767.
#'
#' @examples
#' # Load data
#' data(cancers_genes_drugs, package = "EnrichIntersect")
#'
"cancers_genes_drugs"

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

enrichment_core_original <- function(x, set_indices, alpha = 0.0, normalize = TRUE, permute_n = 100L) {
    .Call(`_EnrichIntersect_enrichment_core_original`, x, set_indices, alpha, normalize, permute_n)
}

#' @keywords internal
#' @aliases EnrichIntersect-package NULL
#'
#'
"_PACKAGE"

## usethis namespace: start
#' @useDynLib EnrichIntersect, .registration = TRUE
#' @importFrom Rcpp sourceCpp
## usethis namespace: end
NULL

#' @title Plot Sankey diagram for intersecting sets
#' @description
#' Plot Sankey diagram for intersecting set through an array
#'
#' @name intersectSankey
#'
#' @importFrom dplyr %>%
#' @importFrom networkD3 sankeyNetwork JS
#' @importFrom htmlwidgets onRender saveWidget
#' @importFrom webshot2 webshot
#' @importFrom methods hasArg
#' @importFrom grDevices hcl.colors
#' @importFrom jsonlite toJSON
#'
#' @param x an array for constructing intersecting set
#' @param out.fig print the figure into \code{"html"}, \code{"pdf"} or 
#' \code{"png"} file. Default is \code{NULL} with R graphics device
#' @param color a vector of colors corresponding to individual tasks
#' @param step.names names of the three dimensions of the array \code{x}, i.e. 
#' names of multiple levels, intermediate variables and tasks. Default is 
#' \code{c("Levels","Variables","Tasks")}. If \code{step.names=NULL}, it will 
#' not show the names
#' @param fontSize a value or vector of three values. If it is one value, it 
#' is the font size for all labels. But a vector of three values specifies the 
#' font size of the labels in the left, middle and right, respectively. Default
#' is \code{c(20,10,20)}
#' @param nodePadding numeric essentially influences the width height
#' @param nodeWidth numeric width of each node
#' @param margin an integer or a named list/vector of integers for the plot 
#' margins
#' @param ... graphics parameters to be passed to \code{sankeyNetwork()} 
#' from R package \code{networkD3}
#'
#' @return An object of a D3 JavaScript intersecting Sankey diagram for 
#' visualising associations based on the input array.
#'
#' @examples
#' # Data set 'cancers_genes_drugs' is an array with association scores between 56 genes (1st
#' # dimension), three cancer types (2nd dimension) and two drugs (3rd dimension)
#' data(cancers_genes_drugs, package = "EnrichIntersect")
#'
#' intersectSankey(cancers_genes_drugs, step.names = c("Cancers", "Genes", "Drugs"))
#'
#' @export
intersectSankey <- function(x, out.fig = NULL, color = NULL, 
                            step.names = c("Levels", "Variables", "Tasks"), 
                            fontSize = c(20, 13, 20), nodePadding = 11,
                            nodeWidth = 5, margin = list(right = 180), ...) {
  # remove intermediate variables without associations with any tasks or levels
  x <- x[apply(x != 0, 1, sum) != 0, , ]
  # remove levels without associations with any intermediate variables or tasks
  x <- x[, apply(x != 0, 2, sum) != 0, ]
  # remove tasks without associations with any intermediate variables or levels
  x <- x[, , apply(x != 0, 3, sum) != 0]

  if(is.null(dimnames(x)[[1]]) | is.null(dimnames(x)[[1]]) | is.null(dimnames(x)[[1]])) {
    stop("Please provide all dimnames of x!")
  }
  # intermediate variables
  inter_var <- dimnames(x)[[1]]
  # multiple sample groups
  multilevel <- dimnames(x)[[2]]
  # multiple response variables
  multitask <- dimnames(x)[[3]]

  # extract nonzero x. We might want to keep weights for edge width in the 
  # diagram in future, but it is not nice for too many variables
  x0 <- array(as.numeric(x != 0), dim = dim(x)) # inter_var, multilevel, multitask
  source <- target <- NULL
  for (i in seq_len(dim(x0)[2])) {
    source <- c(source, rep(multilevel[i], sum(x0[, i, ])))
    target <- c(target, rep(inter_var[rowSums(x0[, i, ]) != 0], times = 
                              rowSums(x0[, i, ])[rowSums(x0[, i, ]) != 0]))
  }
  for (i in seq_len(dim(x0)[1])) {
    source <- c(source, rep(inter_var[i], sum(colSums(x0[i, , ]) != 0)))
    target <- c(target, multitask[colSums(x0[i, , ]) != 0])
  }

  links <- data.frame(source = source, target = target)
  links$value <- rep(.1, nrow(links))

  # construct edge colors
  links$group <- c(rep("white", nrow(links)))
  for (i in seq_along(multitask)) {
    links$group[links$target == multitask[i]] <- multitask[i]
  }
  
  group_tmp <- NULL
  for (i in seq_len(dim(x0)[2])) {
    tmp <- matrix(x0[rowSums(x0[, i, ]) > 0, i, ], ncol = dim(x0)[3])
    group_tmp <- c(group_tmp, unlist(apply(tmp, 1, function(xx) {
      multitask[xx == 1]
    })))
  }
  links$group[seq_along(group_tmp)] <- group_tmp
  links$group <- as.factor(links$group)

  nodes <- data.frame(name = c(as.character(links$source), 
                               as.character(links$target)) %>% unique())
  links$IDsource <- match(links$source, nodes$name) - 1
  links$IDtarget <- match(links$target, nodes$name) - 1
  nodes$group <- as.factor(c("my_unique_group"))
  nodes$targetL <- 
    c(rep(TRUE, dim(x)[2]), rep(FALSE, length(nodes$name) - dim(x)[2]))
  nodes$targetR <- 
    c(rep(FALSE, length(nodes$name) - dim(x)[3]), rep(TRUE, dim(x)[3]))
  nodes$targetM <- 
    c(rep(FALSE, dim(x)[2]), rep(TRUE, dim(x)[1]), rep(FALSE, dim(x)[3]))

  # put colors for different tasks in a data.frame
  if (is.null(color)) {
    mycolor <- c(hcl.colors(dim(x)[3], palette = "Dynamic"), "white")
  } else {
    if (length(color) != dim(x)[3]) {
      stop("Please specify the argument 'colors' correctly!")
    }
    mycolor <- c(color, "white")
  }
  color_scale <- data.frame(
    range = mycolor,
    domain = c(paste("Task", seq_len(dim(x)[3]), sep = ""), "my_unique_group"),
    stringsAsFactors = FALSE
  )
  if (!(length(fontSize) == 1 || length(fontSize) == 3)) {
    stop("Please specify the argument 'fontSize' correctly!")
  }
  if (length(fontSize) == 1) {
    fontSize <- rep(fontSize, 3)
  }

  # # check and set some default arguments passing to sankeyNetwork()
  # if (!hasArg(nodePadding)) nodePadding <- 11
  # if (!hasArg(nodeWidth)) nodeWidth <- 5
  # if (!hasArg(margin)) margin <- list(right = 180)

  # plot a Sankey diagram
  g <- sankeyNetwork(
    Links = links, Nodes = nodes, Source = "IDsource", Target = "IDtarget", 
    Value = "value", NodeID = "name", sinksRight = FALSE, 
    fontSize = fontSize[1], nodePadding = nodePadding, nodeWidth = nodeWidth,
    colourScale = JS(
      sprintf(
        "d3.scaleOrdinal() .domain(%s) .range(%s)",
        toJSON(color_scale$domain),
        toJSON(color_scale$range)
      )
    ),
    LinkGroup = "group", NodeGroup = "group", margin = margin, ...)
  # push left labels to the left of the nodes
  g$x$nodes$targetL <- nodes$targetL
  g$x$nodes$targetM <- nodes$targetM
  g$x$nodes$targetR <- nodes$targetR

  # change font sizes of the node labels on the left and right via widget's built-in JavaScript
  df <- data.frame(
    fontsize = fontSize, # paste(fontSize, "px", sep=""),
    stringsAsFactors = FALSE
  )
  if (is.null(step.names)) {
    df$step <- rep("", length(fontSize))
    df$stepSize <- rep(0, length(fontSize))
  } else {
    df$step <- c(step.names, rep(NA, length(fontSize) - 3))
    df$stepSize <- c(max(fontSize), rep(NA, length(fontSize) - 1))
    df$stepAlign <- c(-11, 0, 5)
  }

  g <- onRender(g, '
  function(el, x, data) {
    // var fontsize1 = fontsize1;
    // set label font size

    d3.select(el)
      .selectAll(".node text")
      .filter(d => d.targetL)
      .attr("x", -10 + x.options.nodeWidth) // or use -6
      .attr("text-anchor", "end")
      .style("font-size", data.fontsize[0]);

    d3.select(el)
      .selectAll(".node text")
      .filter(d => d.targetM)
      .style("font-size", data.fontsize[1]);

    d3.select(el)
      .selectAll(".node text")
      .filter(d => d.targetR)
      .style("font-size", data.fontsize[2]);

      // set x-axis labels
    var cols_x = this.sankey.nodes()
      .map(d => d.x).filter((v, i, a) => a.indexOf(v) === i)
      .sort(function(a, b){return a - b});

    cols_x.forEach((d, i) => {
      d3.select(el).select("svg")
        .append("text")
        .attr("x", d+data.stepAlign[i]*x.options.nodeWidth)
        .attr("y", 3)
        .attr("font-weight", "bold")
        .text(data.step[i])
        .style("font-size", data.stepSize[0]);
    })
  }
  ', data = df)

  # output figure
  if (!is.null(out.fig)) {
    if (out.fig %in% c("html", "pdf", "png")) {
      saveWidget(widget = g, file = "sankey.html")
    }
    if (out.fig == "pdf") {
      webshot(url = "sankey.html", file = "sankey.pdf")
    }
    if (out.fig == "png") {
      webshot(url = "sankey.html", file = "sankey.png")
    }
  } else {
    g
  }
}

#' @title Data set 'cancers_drug_groups'
#'
#' @description
#' Data set 'cancers_drug_groups' is a list including a score dataframe with 
#' 147 drugs as rows and 19 cancer types as columns, and a dataframe with 9 
#' self-defined drug groups (1st column) of the 147 drugs (2nd column). More 
#' details of the data can be found in the results of paper 'Tissue-specific 
#' identification of multi-omics features for pan-cancer drug response 
#' prediction' by Zhao Z., Wang S. Zucknick M. and Aittokallio T. (2022).
#'
#' @return No return value.
#'
#' @references Zhao Z, Wang S, Zucknick M and Aittokallio T (2022). 
#' \emph{Tissue-specific identification of multi-omics features for 
#' pan-cancer drug response prediction}. iScience, 25(8): 104767.
#'
#' @examples
#' # Load data
#' data(cancers_drug_groups, package = "EnrichIntersect")
#'
"cancers_drug_groups"

---
title: "EnrichIntersect Tutorial"
author: "Zhi Zhao"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{EnrichIntersect}
  \usepackage[utf8]{inputenc}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```


This is a flexible tool for enrichment analysis based on user-defined sets. It allows users to perform over-representation analysis of the custom sets among any specified ranked feature list, hence making enrichment analysis applicable to various types of data from different scientific fields. EnrichIntersect also enables an interactive means to visualize identified associations based on, for example, the mix-lasso model ([Zhao et al., 2022](https://doi.org/10.1016/j.isci.2022.104767)) or similar methods.

To get started, load the package with

```{r, eval=TRUE}
library("EnrichIntersect")
```

## Examples


### Plot enrichment map

An example input data object `cancers_drug_groups` is an R `list` provided in our package, which includes a `data.frame` object with 147 cancer drugs as rows and nine cancer types as columns, and another `data.frame` that groups the 147 drugs (second column) into nine user-defined drug classes (first column). The default setup of `enrichment()` uses the classic K-S test statistic to calculate the normalized enrichment score that quantifies the degree to which the features in a feature set are over-represented at the top or bottom of the entire ranked list of features (e.g. a list of drugs), using as default 100 permutations for the empirical null test statistic. In the visualization, the statistically significantly enriched feature sets are marked with red coloured circles given a pre-specified significance level. The pre-specified significance level will be adjusted automatically if argument `padj.method` is one of `c("holm","hochberg","hommel","bonferroni","BH","BY","fdr","none")`. Users can specify the argument `alpha` for calculating a weighted enrichment score, argument `normalize=FALSE` for using the standard enrichment score rather than the normalized score, argument `permute.n` for the number of permutations of the ranked feature list used for estimating the empirical null test statistic, and the argument `pvalue.cutoff` for marking enriched categories at a specific significance level. See the following code for an example.

```{r enrichment_map, results="hide", fig.width=7.2, fig.height=5}
data(cancers_drug_groups, package = "EnrichIntersect")
x <- cancers_drug_groups$score
custom.set <- cancers_drug_groups$custom.set
set.seed(123)
enrich <- enrichment(x, custom.set, permute.n = 100)
```


### Plot Sankey diagram for intersecting set through an array

`EnrichIntersect` function `intersectSankey()` creates a Sankey diagram to visualize intersecting sets from an `array` object, in which the first dimension represents intermediate variables, and the second and third dimensions represent multiple levels and multiple tasks, respectively. One intersecting set is a list of intermediate variables associated with a combination of a subset of levels and a subset of tasks, which is not easy to visualize when all possible combinations of the two are many. Our function `intersectSankey()` has adapted `sankeyNetwork()` from R package `networkD3` to create a `D3' `JavaScript' interactive Sankey diagram in order to be suitable for several levels, multiple tasks and many intermediate variables. Besides saving the Sankey diagram as an interactive html file, similarly to `networkD3`, the user can also save the Sankey diagram as a pdf or png file via R package `webshot2`. The argument `out.fig=c(NA,"html","pdf","png")` in the function `intersectSankey()` determines the figure on the user's R graphics device, to be saved either as a html, pdf or png file. 

An example input data object `cancers_genes_drugs` in the package is an `array` with associations between, e.g., 56 genes (first dimension), two cancer types (second dimension) and two drugs (third dimension) provided in our package. The user can adjust the Sankey diagram argument `out.fig` for different output graph types  and use argument `step.names` to indicate the labels of the three kinds of variables in a Sankey diagram, i.e., name of multiple levels, name of intermediate variables, and name of multiple tasks, see the following code for an example.

```{r sankey_diagram, fig.width=9, fig.height=7}
data(cancers_genes_drugs, package = "EnrichIntersect")
# better to use argument `out.fig = "pdf"` for printing a pdf or html figure
intersectSankey(cancers_genes_drugs, step.names = c("Cancers", "Genes", "Drugs"))
```


## Citation

> Zhi Zhao, Manuela Zucknick, Tero Aittokallio (2022).
> EnrichIntersect: an R package for custom set enrichment analysis and interactive visualization of intersecting sets. 
> _Bioinformatics Advances_, 2(1), vbac073. DOI: [10.1093/bioadv/vbac073](https://doi.org/10.1093/bioadv/vbac073).

> Zhi Zhao, Shixiong Wang, Manuela Zucknick, Tero Aittokallio (2022).
> Tissue-specific identification of multi-omics features for pan-cancer drug response prediction.
> _iScience_, 25(8): 104767. DOI: [10.1016/j.isci.2022.104767](https://doi.org/10.1016/j.isci.2022.104767).

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrichment.R
\name{enrichment}
\alias{enrichment}
\title{Plot enrichment map}
\usage{
enrichment(
  x,
  custom.set,
  alpha = 0,
  normalize = TRUE,
  permute.n = 100,
  padj.method = "none",
  pvalue.cutoff = 0.05,
  angle = 45,
  match.feature = c("rownames", "colnames"),
  ...
)
}
\arguments{
\item{x}{a vector or matrix of scores to be enriched}

\item{custom.set}{a self-defined set. The first column contains feature names,
and the second column, preferably named "group", contains group names.}

\item{alpha}{exponent weight of the score of ordered features.}

\item{normalize}{logical value to determine if normalizing enrichment scores.}

\item{permute.n}{number of custom-set permutations for significance testing.}

\item{padj.method}{correction method passed to stats::p.adjust.}

\item{pvalue.cutoff}{cutoff for both unadjusted and adjusted p-value.}

\item{angle}{angle of rotating x-axis labels.}

\item{match.feature}{one of "rownames" or "colnames". Default is "rownames",
which keeps the original EnrichIntersect behavior.}

\item{...}{other arguments}
}
\value{
A list including S, pvalue, and g.
}
\description{
Plot enrichment map through a vector or matrix of scores and a self-defined
set that summarizes a few groups of the names of the vector or matrix.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cancers_drug_groups.R
\docType{data}
\name{cancers_drug_groups}
\alias{cancers_drug_groups}
\title{Data set 'cancers_drug_groups'}
\format{
An object of class \code{list} of length 2.
}
\usage{
cancers_drug_groups
}
\value{
No return value.
}
\description{
Data set 'cancers_drug_groups' is a list including a score dataframe with 
147 drugs as rows and 19 cancer types as columns, and a dataframe with 9 
self-defined drug groups (1st column) of the 147 drugs (2nd column). More 
details of the data can be found in the results of paper 'Tissue-specific 
identification of multi-omics features for pan-cancer drug response 
prediction' by Zhao Z., Wang S. Zucknick M. and Aittokallio T. (2022).
}
\examples{
# Load data
data(cancers_drug_groups, package = "EnrichIntersect")

}
\references{
Zhao Z, Wang S, Zucknick M and Aittokallio T (2022). 
\emph{Tissue-specific identification of multi-omics features for 
pan-cancer drug response prediction}. iScience, 25(8): 104767.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/intersectSankey.R
\name{intersectSankey}
\alias{intersectSankey}
\title{Plot Sankey diagram for intersecting sets}
\usage{
intersectSankey(
  x,
  out.fig = NULL,
  color = NULL,
  step.names = c("Levels", "Variables", "Tasks"),
  fontSize = c(20, 13, 20),
  nodePadding = 11,
  nodeWidth = 5,
  margin = list(right = 180),
  ...
)
}
\arguments{
\item{x}{an array for constructing intersecting set}

\item{out.fig}{print the figure into \code{"html"}, \code{"pdf"} or 
\code{"png"} file. Default is \code{NULL} with R graphics device}

\item{color}{a vector of colors corresponding to individual tasks}

\item{step.names}{names of the three dimensions of the array \code{x}, i.e. 
names of multiple levels, intermediate variables and tasks. Default is 
\code{c("Levels","Variables","Tasks")}. If \code{step.names=NULL}, it will 
not show the names}

\item{fontSize}{a value or vector of three values. If it is one value, it 
is the font size for all labels. But a vector of three values specifies the 
font size of the labels in the left, middle and right, respectively. Default
is \code{c(20,10,20)}}

\item{nodePadding}{numeric essentially influences the width height}

\item{nodeWidth}{numeric width of each node}

\item{margin}{an integer or a named list/vector of integers for the plot 
margins}

\item{...}{graphics parameters to be passed to \code{sankeyNetwork()} 
from R package \code{networkD3}}
}
\value{
An object of a D3 JavaScript intersecting Sankey diagram for 
visualising associations based on the input array.
}
\description{
Plot Sankey diagram for intersecting set through an array
}
\examples{
# Data set 'cancers_genes_drugs' is an array with association scores between 56 genes (1st
# dimension), three cancer types (2nd dimension) and two drugs (3rd dimension)
data(cancers_genes_drugs, package = "EnrichIntersect")

intersectSankey(cancers_genes_drugs, step.names = c("Cancers", "Genes", "Drugs"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cancers_genes_drugs.R
\docType{data}
\name{cancers_genes_drugs}
\alias{cancers_genes_drugs}
\title{Data set 'cancers_genes_drugs'}
\format{
An object of class \code{array} of dimension 56 x 3 x 2.
}
\usage{
cancers_genes_drugs
}
\value{
No return value.
}
\description{
Data set 'cancers_genes_drugs' is an array with association scores between 
56 genes (1st dimension), three cancer types (2nd dimension) and two drugs 
(3rd dimension). More details of the data can be found in the results of 
paper 'Tissue-specific identification of multi-omics features for pan-cancer 
drug response prediction' by Zhao Z., Wang S. Zucknick M. and Aittokallio 
T. (2022).
}
\examples{
# Load data
data(cancers_genes_drugs, package = "EnrichIntersect")

}
\references{
Zhao Z, Wang S, Zucknick M and Aittokallio T (2022). \emph{
Tissue-specific identification of multi-omics features for pan-cancer drug 
response prediction}. iScience, 25(8): 104767.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EnrichIntersect-package.R
\docType{package}
\name{EnrichIntersect-package}
\alias{EnrichIntersect}
\alias{EnrichIntersect-package}
\title{EnrichIntersect: Enrichment Analysis and Intersecting Sankey Diagram}
\description{
A flexible tool for enrichment analysis based on user-defined sets. It allows users to perform over-representation analysis of the custom sets among any specified ranked feature list, hence making enrichment analysis applicable to various types of data from different scientific fields. 'EnrichIntersect' also enables an interactive means to visualize identified associations based on, for example, the mix-lasso model (Zhao et al., 2022 \doi{10.1016/j.isci.2022.104767}) or similar methods.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/ocbe-uio/EnrichIntersect}
  \item Report bugs at \url{https://github.com/ocbe-uio/EnrichIntersect/issues}
}

}
\author{
\strong{Maintainer}: Zhi Zhao \email{zhi.zhao@medisin.uio.no}

Authors:
\itemize{
  \item Zhi Zhao \email{zhi.zhao@medisin.uio.no}
  \item Manuela Zucknick \email{manuela.zucknick@medisin.uio.no}
}

Other contributors:
\itemize{
  \item Tero Aittokallio \email{t.a.aittokallio@medisin.uio.no} [contributor]
}

}
\keyword{internal}
